@@ -23,33 +23,23 @@ class auxlib
   {
   public:
   
-  
-  template<const uword row, const uword col>
-  struct pos
-    {
-    static constexpr uword n2 = row + col*2;
-    static constexpr uword n3 = row + col*3;
-    static constexpr uword n4 = row + col*4;
-    };
-  
-  
   //
   // inv
   
   template<typename eT>
-  inline static bool inv(Mat<eT>& out, const Mat<eT>& A);
+  inline static bool inv(Mat<eT>& A);
   
   template<typename eT>
-  arma_cold inline static bool inv_tiny(Mat<eT>& out, const Mat<eT>& X);
-  
-  template<typename eT, typename T1>
-  inline static bool inv_tr(Mat<eT>& out, const Base<eT,T1>& X, const uword layout);
-  
-  template<typename eT, typename T1>
-  inline static bool inv_sympd(Mat<eT>& out, const Base<eT,T1>& X);
+  inline static bool inv(Mat<eT>& out, const Mat<eT>& X);
   
   template<typename eT>
-  arma_cold inline static bool inv_sympd_tiny(Mat<eT>& out, const Mat<eT>& X);
+  inline static bool inv_tr(Mat<eT>& A, const uword layout);
+  
+  template<typename eT>
+  inline static bool inv_sympd(Mat<eT>& A);
+  
+  template<typename eT>
+  inline static bool inv_sympd(Mat<eT>& out, const Mat<eT>& X);
   
   template<typename eT>
   inline static bool inv_sympd_rcond(Mat<eT>& A, const eT rcond_threshold);
@@ -59,23 +49,16 @@ class auxlib
   
   
   //
-  // det
+  // det and log_det
   
   template<typename eT>
-  inline static eT det(const Mat<eT>& A);
+  inline static bool det(eT& out_val, Mat<eT>& A);
   
   template<typename eT>
-  arma_cold inline static eT det_tinymat(const Mat<eT>& X, const uword N);
+  inline static bool log_det(eT& out_val, typename get_pod_type<eT>::result& out_sign, Mat<eT>& A);
   
   template<typename eT>
-  inline static eT det_lapack(const Mat<eT>& X);
-  
-  
-  //
-  // log_det
-  
-  template<typename eT, typename T1>
-  inline static bool log_det(eT& out_val, typename get_pod_type<eT>::result& out_sign, const Base<eT,T1>& X);
+  inline static bool log_det_sympd(typename get_pod_type<eT>::result& out_val, Mat<eT>& A);
   
   
   //
@@ -191,6 +174,9 @@ class auxlib
   template<typename eT>
   inline static bool chol_band_common(Mat<eT>& X, const uword KD, const uword layout);
   
+  template<typename eT>
+  inline static bool chol_pivot(Mat<eT>& X, Mat<uword>& P, const uword layout);
+  
   
   //
   // hessenberg decomposition
