@@ -21,6 +21,7 @@
 
 
 class op_all
+  : public traits_op_xvec
   {
   public:
   
@@ -45,8 +46,8 @@ class op_all
   all_vec_helper
     (
     const mtOp<uword, T1, op_type>& X,
-    const typename arma_op_rel_only<op_type>::result           junk1 = 0,
-    const typename arma_not_cx<typename T1::elem_type>::result junk2 = 0
+    const typename arma_op_rel_only<op_type>::result*           junk1 = nullptr,
+    const typename arma_not_cx<typename T1::elem_type>::result* junk2 = nullptr
     );
   
   
@@ -55,9 +56,9 @@ class op_all
   all_vec_helper
     (
     const mtGlue<uword, T1, T2, glue_type>& X,
-    const typename arma_glue_rel_only<glue_type>::result       junk1 = 0,
-    const typename arma_not_cx<typename T1::elem_type>::result junk2 = 0,
-    const typename arma_not_cx<typename T2::elem_type>::result junk3 = 0
+    const typename arma_glue_rel_only<glue_type>::result*       junk1 = nullptr,
+    const typename arma_not_cx<typename T1::elem_type>::result* junk2 = nullptr,
+    const typename arma_not_cx<typename T2::elem_type>::result* junk3 = nullptr
     );
   
   
