@@ -43,11 +43,11 @@ class op_median
   {
   public:
   
-  template<typename T1>
-  inline static void apply(Mat<typename T1::elem_type>& out, const Op<T1,op_median>& in);
+  template<typename eT, typename T1>
+  inline static void apply(Mat<eT>& out, const Op<T1,op_median>& in, const typename arma_not_cx<eT>::result* junk = nullptr);
   
-  template<typename T, typename T1>
-  inline static void apply(Mat< std::complex<T> >& out, const Op<T1,op_median>& in);
+  template<typename eT, typename T1>
+  inline static void apply(Mat<eT>& out, const Op<T1,op_median>& in, const typename arma_cx_only<eT>::result* junk = nullptr);
   
   //
   //
